"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Curried wrappers for building type guards
 */
exports.is = function (fn) { return function (value) { return fn(value); }; };
exports.isEq = function (x) { return exports.is(function (y) { return y === x; }); };
/**
 * Generic type guards
 */
exports.isString = exports.is(function (value) { return typeof value === 'string'; });
exports.isNumber = exports.is(function (value) { return typeof value === 'number'; });
exports.isBoolean = exports.is(function (value) { return value === true || value === false; });
exports.isSymbol = exports.is(function (value) { return typeof value === 'symbol'; });
exports.isFunction = exports.is(function (value) { return typeof value === 'function'; });
exports.isArray = exports.is(Array.isArray);
exports.isShape = exports.is(function (value) { return typeof value === 'object' && !exports.isFunction(value) && !exports.isArray(value) && value !== null; });
exports.isNull = exports.isEq(null);
exports.isUndefined = exports.isEq(undefined);
exports.hasKey = function (k, x) {
    return !!(x && !exports.isUndefined(x[k]));
};
/** @internal
 * Returns a string value for any type
 */
exports.toString = function (val) {
    return exports.isString(val)
        ? val
        : exports.isShape(val) || exports.isArray(val)
            ? exports.errorJ(val)
            : exports.isNull(val)
                ? 'null'
                : exports.isUndefined(val)
                    ? 'undefined'
                    : val.toString();
};
exports.errorT = function (expected, received) {
    return "Expected " + expected + " Recieved " + exports.toString(received);
};
exports.errorJ = function (x) {
    return JSON.stringify(exports.isArray(x) ? x.map(function (x, i) { return "[" + i + "] " + exports.toString(x); }) : x, function (_key, value) { return (exports.isString(value) ? value.replace(/\n/gim, "\n  ") : value); }, 2)
        .replace(/"/gim, '')
        .replace(/\\n/gim, "\n");
};
/**
 * TS Prove type guards
 */
exports.isError = exports.is(function (x) { return exports.isString(x && x[0]); });
//# sourceMappingURL=utils.js.map