"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("./utils");
/** @internal
 * Validator result wrapper
 */
exports.result = function (x, err, res) {
    return x ? [null, res] : [err, res];
};
/**
 * TS Prove validators
 */
exports._string = function () { return function (x) { return exports.result(utils_1.isString(x), utils_1.errorT('string', x), x); }; };
exports._number = function () { return function (x) { return exports.result(utils_1.isNumber(x), utils_1.errorT('number', x), x); }; };
exports._boolean = function () { return function (x) { return exports.result(utils_1.isBoolean(x), utils_1.errorT('boolean', x), x); }; };
exports._symbol = function () { return function (x) { return exports.result(utils_1.isSymbol(x), utils_1.errorT('symbol', x), x); }; };
exports._null = function () { return function (x) { return exports.result(utils_1.isNull(x), utils_1.errorT('null', x), x); }; };
exports._undefined = function () { return function (x) {
    return exports.result(utils_1.isUndefined(x), utils_1.errorT('undefined', x), x);
}; };
exports._any = function () { return function (x) { return exports.result(true, '', x); }; };
exports._unknown = function () { return function (x) {
    return exports.result(true, '', x);
}; };
exports._constant = function (x) { return function (v) {
    return exports.result(v === x, x + " is not equal to " + utils_1.toString(v), v);
}; };
/**
 * Determin that type is on of an array of types and return an intersection type
 * @param P Prove<T>[]
 * @return (v: unknown) => Success<T[number]> | Failure
 */
exports._or = function (x) { return function (y) {
    return exports.result(x.reduce(function (a, b) { return a === true || !b(y)[0]; }, false) === true, "Nothing matched " + utils_1.toString(y), y);
}; };
/**
 * Determin that every element in an array is of type
 * @param P Prove<T>
 * @return (v: unknown) => Success<T[]> | Failure
 */
exports._array = function (x) { return function (y) {
    if (!utils_1.isArray(y))
        return [utils_1.errorT('array', y), y];
    var result = y.reduce(function (a, b) { return (utils_1.isError(x(b)) ? __spreadArrays(a, [x(b)[0]]) : a); }, []);
    if (result.length > 0)
        return [utils_1.errorJ(result), y];
    return [null, y];
}; };
/**
 * Execute a key value object of validators
 * @param P { [x: string]: Prove<T> }
 * @return (v: unknown) => Success<{ [x: string]: T }> | Failure
 */
exports._shape = function (shpe) { return function (y) {
    var result = Object.keys(shpe).reduce(function (all, key) {
        var _a, _b;
        if (!utils_1.hasKey(key, y))
            return [true, __assign(__assign({}, all[1]), (_a = {}, _a[key] = '__missing__', _a))];
        var res = shpe[key](y[key]);
        if (utils_1.isError(res))
            return [true, __assign(__assign({}, all[1]), (_b = {}, _b[key] = res[0], _b))];
        return all;
    }, [false, {}]);
    if (result[0])
        return [utils_1.errorJ(result[1]), y];
    return [null, y];
}; };
//# sourceMappingURL=validators.js.map