/**
 * Return types
 */
export declare type Failure<T extends any> = [string, T];
export declare type Success<T extends any> = [null, T];
export declare type Proof<T extends any> = Success<T> | Failure<unknown>;
/**
 * Generic wrap and unwrap for validators
 */
export declare type Wrap<P extends any, U extends any = any> = (x: U) => Failure<U> | Success<P>;
export declare type Unwrap<T extends Prove> = T extends Wrap<infer A> ? A : never;
/**
 * Validators
 */
export declare namespace P {
    type String = Wrap<string>;
    type Number = Wrap<number>;
    type Boolean = Wrap<boolean>;
    type Symbol = Wrap<symbol>;
    type Null = Wrap<null>;
    type Undefined = Wrap<undefined>;
    type Infer<T extends any> = Wrap<T>;
    type Or<T extends Prove[]> = Wrap<Unwrap<T[number]>>;
    type Array<T extends Prove> = Wrap<Unwrap<T>[]>;
    type Shape<T extends {
        [x: string]: Prove;
    }> = Wrap<{
        [Key in keyof T]: Unwrap<T[Key]>;
    }>;
}
/**
 * Validator unions
 */
export declare type ProvePrimitive = P.Number | P.String | P.Symbol | P.Boolean | P.Null | P.Undefined;
export declare type ProveStructure = P.Array<any> | P.Shape<any> | P.Or<any> | P.Infer<any>;
export declare type Prove = ProvePrimitive | ProveStructure;
