import { Failure } from './types';
/**
 * Curried wrappers for building type guards
 */
export declare const is: <T>(fn: (x: unknown) => boolean) => (value: unknown) => value is T;
export declare const isEq: <T>(x: T) => (value: unknown) => value is T;
/**
 * Generic type guards
 */
export declare const isString: (value: unknown) => value is string;
export declare const isNumber: (value: unknown) => value is number;
export declare const isBoolean: (value: unknown) => value is boolean;
export declare const isSymbol: (value: unknown) => value is symbol;
export declare const isFunction: (value: unknown) => value is Function;
export declare const isArray: (value: unknown) => value is unknown[];
export declare const isShape: (value: unknown) => value is Record<string, any>;
export declare const isNull: (value: unknown) => value is null;
export declare const isUndefined: (value: unknown) => value is undefined;
export declare const hasKey: <K extends any>(k: K, x: unknown) => x is Record<K, any>;
/** @internal
 * Returns a string value for any type
 */
export declare const toString: (val: unknown) => string;
export declare const errorT: (expected: string, received: any) => string;
export declare const errorJ: <T extends any[] | Record<string, string>>(x: T) => string;
/**
 * TS Prove type guards
 */
export declare const isError: (value: unknown) => value is Failure<unknown>;
