import { Prove, Success, Failure, Unwrap } from './types';
/** @internal
 * Validator result wrapper
 */
export declare const result: <T>(x: boolean, err: string, res: T) => Success<T> | Failure<unknown>;
/**
 * TS Prove validators
 */
export declare const _string: () => import("./types").Wrap<string, any>;
export declare const _number: () => import("./types").Wrap<number, any>;
export declare const _boolean: () => import("./types").Wrap<boolean, any>;
export declare const _symbol: () => import("./types").Wrap<symbol, any>;
export declare const _null: () => import("./types").Wrap<null, any>;
export declare const _undefined: () => import("./types").Wrap<undefined, any>;
export declare const _any: <T extends any = any>() => import("./types").Wrap<T, any>;
export declare const _unknown: <T extends unknown = unknown>() => import("./types").Wrap<T, any>;
export declare const _constant: <T extends string>(x: T) => import("./types").Wrap<T, any>;
/**
 * Determin that type is on of an array of types and return an intersection type
 * @param P Prove<T>[]
 * @return (v: unknown) => Success<T[number]> | Failure
 */
export declare const _or: <T extends Prove[]>(x: T) => import("./types").Wrap<Unwrap<T[number]>, any>;
/**
 * Determin that every element in an array is of type
 * @param P Prove<T>
 * @return (v: unknown) => Success<T[]> | Failure
 */
export declare const _array: <T extends Prove>(x: T) => import("./types").Wrap<Unwrap<T>[], any>;
/**
 * Execute a key value object of validators
 * @param P { [x: string]: Prove<T> }
 * @return (v: unknown) => Success<{ [x: string]: T }> | Failure
 */
export declare const _shape: <T extends {
    [x: string]: Prove;
}>(shpe: T) => import("./types").Wrap<{ [Key in keyof T]: Unwrap<T[Key]>; }, any>;
